/*================================================================
  File:     01_CapturingQueries.sql

  Summary:  Queries used to show the various Showplan methods

  Latest SQL Server Versions tested: SQL Server 2014 12.0.4100
------------------------------------------------------------
  Modified by Erin Stellato, SQLskills.com

  (c) 2015, SQLskills.com. All rights reserved.

  For more scripts and sample code, check out 
    http://www.SQLskills.com

  You may alter this code for your own *non-commercial* purposes. You may
  republish altered code as long as you include this copyright and give due
  credit, but you must obtain prior permission before blogging this code.
  
  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF 
  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED 
  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
  PARTICULAR PURPOSE.
=================================================================*/

/*
	Clear cache - not for production use :(
*/
DBCC FREEPROCCACHE;
GO

USE [AdventureWorks2014];
GO

/*
	Display Estimated Graphical Showplan
*/
SELECT
[d].[SalesOrderDetailID], [d].[ProductID], [h].[CustomerID], [h].[TerritoryID]
FROM [Sales].[SalesOrderDetail] [d]
JOIN [Sales].[SalesOrderHeader] [h] on [d].[SalesOrderID] = [h].[SalesOrderID]
WHERE [h].[SalesPersonID] IS NULL;
GO


/*
	Display Actual Graphical Showplan
	For each operator get estimated and actual information
	tool tip is a subset of information...F4 is your friend
	SELECT - memory grant info, estimated subtree cost 
*/

SELECT
[d].[SalesOrderDetailID], [d].[ProductID], [h].[CustomerID], [h].[TerritoryID]
FROM [Sales].[SalesOrderDetail] [d]
JOIN [Sales].[SalesOrderHeader] [h] on [d].[SalesOrderID] = [h].[SalesOrderID]
WHERE [h].[SalesPersonID] IS NULL;
GO



/*
	XML Showplan *Estimated*
	can see operations in XML format, can search (CTRL+F)
*/

SET SHOWPLAN_XML ON;
GO

SELECT
	[t].[TransactionID],
	[t].[ProductID],
	[h].[OrderDate],
	[h].[CustomerID],
	[h].[TotalDue]
FROM [Production].[TransactionHistory] [t]
JOIN [Sales].[SalesOrderHeader] [h] on [t].[ReferenceOrderID] = [h].[SalesOrderID];
GO

SET SHOWPLAN_XML OFF;
GO


/*
	XML Showplan Actual
	Includes run-time statisics
*/
SET STATISTICS XML ON;
GO

SELECT
[d].[SalesOrderDetailID], [d].[ProductID], [h].[CustomerID], [h].[TerritoryID]
FROM [Sales].[SalesOrderDetail] [d]
JOIN [Sales].[SalesOrderHeader] [h] on [d].[SalesOrderID] = [h].[SalesOrderID]
WHERE [h].[SalesPersonID] IS NULL;
GO

SET STATISTICS XML OFF;
GO


/*
	Save and open a saved .sqlplan file

	View in SQLSentry Plan Explorer

	Copy XML into PE

	Viewing the schema:
	http://schemas.microsoft.com/sqlserver/2004/07/showplan/

*/



/*
	Get a plan from cache...
	clear cache first - still not for production use :(
*/
DBCC FREEPROCCACHE;
GO


/*
	execute a query
*/
SELECT  *
FROM [Sales].[SalesOrderHeader]
WHERE [TotalDue] > 10000;
GO


/*
	check the DMV
*/
SELECT	[size_in_bytes],
		[cacheobjtype],
		[objtype],
		[plan_handle]
FROM [sys].[dm_exec_cached_plans];


/*
	find plan based on query text
*/
SELECT	[cp].[size_in_bytes],
		[cp].[cacheobjtype],
		[cp].[objtype],
		[cp].[plan_handle],
		[dest].[text]
FROM [sys].[dm_exec_cached_plans] [cp]
CROSS APPLY [sys].[dm_exec_sql_text]([cp].[plan_handle]) [dest]
WHERE [dest].[text] LIKE '%SalesOrderHeader%';


/*
	get the plan
*/
SELECT	[dbid],
		[query_plan]
FROM [sys].[dm_exec_query_plan](0x060007004B18E030900225FF0000000001000000000000000000000000000000000000000000000000000000)


/*
	query dm_exec_text_query_plan for large plans
	not clickable link in output
	how to view easily?
*/
SELECT	[dbid],
		[query_plan]
FROM [sys].[dm_exec_text_query_plan](
0x060007004B18E030900225FF0000000001000000000000000000000000000000000000000000000000000000
, 0, -1)


/*
	Clear cache again - really, it's not for production use :(
*/
DBCC FREEPROCCACHE;
GO


/*
	Display Estimated Graphical Showplan
*/
SELECT
[d].[SalesOrderDetailID], [d].[ProductID], [h].[CustomerID], [h].[TerritoryID]
FROM [Sales].[SalesOrderDetail] [d]
JOIN [Sales].[SalesOrderHeader] [h] on [d].[SalesOrderID] = [h].[SalesOrderID]
WHERE [h].[SalesPersonID] IS NULL;
GO


/*
	prove it's not actually cached...
*/
SELECT	[cp].[size_in_bytes],
		[cp].[cacheobjtype],
		[cp].[objtype],
		[cp].[plan_handle],
		[dest].[text]
FROM [sys].[dm_exec_cached_plans] [cp]
CROSS APPLY [sys].[dm_exec_sql_text]([cp].[plan_handle]) [dest]
WHERE [dest].[text] LIKE '%SalesOrderHeader%';

/*
	Show XE event warning
	(this event session is not recommended)
*/	
CREATE EVENT SESSION [Capture_Plans] 
	ON SERVER 
ADD EVENT	
	sqlserver.query_post_execution_showplan 
ADD TARGET 
	package0.event_file(SET filename=N'C:\temp\Capture_Plans')
WITH (
	MAX_MEMORY=4096 KB,EVENT_RETENTION_MODE=ALLOW_SINGLE_EVENT_LOSS,
	MAX_DISPATCH_LATENCY=30 SECONDS,MAX_EVENT_SIZE=0 KB,MEMORY_PARTITION_MODE=NONE,
	TRACK_CAUSALITY=OFF,STARTUP_STATE=OFF)
GO


/*
	Drop event session when done
*/
DROP EVENT SESSION [Capture_Plans] 
	ON SERVER ;
GO






/*
	Deprecated options...
*/


/*
	SHOWPLAN_TEXT (estimated)
	statement text and each operation
*/
SET SHOWPLAN_TEXT ON;
GO

SELECT [SalesOrderID]
FROM [Sales].[SalesOrderHeaderSalesReason]
WHERE [ModifiedDate] = '2005-07-10 00:00:00.000';
GO

SET SHOWPLAN_TEXT OFF;
GO


/*
	SHOWPLAN_TEXT (estimated)
	operations and additional info
*/
SET SHOWPLAN_ALL ON;
GO

SELECT [SalesOrderID]
FROM [Sales].[SalesOrderHeaderSalesReason]
WHERE [ModifiedDate] = '2005-07-10 00:00:00.000';
GO

SET SHOWPLAN_ALL OFF;
GO


/*
	STATISTICS PROFILE (actual)
	operations and additional info - like SHOWPLAN_ALL
	also get run-time statistics
*/
SET STATISTICS PROFILE ON;
GO

SELECT [SalesOrderID]
FROM [Sales].[SalesOrderHeaderSalesReason]
WHERE [ModifiedDate] = '2005-07-10 00:00:00.000';
GO

SET STATISTICS PROFILE OFF;
GO