/*================================================================
  File:     03_MoreInformation.sql

  Summary:  Queries to explore additional information in plans

  Latest SQL Server Versions tested: SQL Server 2014 12.0.4100
------------------------------------------------------------
  Modified by Erin Stellato, SQLskills.com

  (c) 2015, SQLskills.com. All rights reserved.

  For more scripts and sample code, check out 
    http://www.SQLskills.com

  You may alter this code for your own *non-commercial* purposes. You may
  republish altered code as long as you include this copyright and give due
  credit, but you must obtain prior permission before blogging this code.
  
  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF 
  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED 
  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
  PARTICULAR PURPOSE.
=================================================================*/

USE [AdventureWorks2014];
GO

/*
	Missing index notification
*/
SELECT [h].[SalesOrderID], [d].[ProductID], [d].[OrderQty], [d].[UnitPrice]
FROM [Sales].[SalesOrderDetail] AS [d]
INNER JOIN [Sales].[SalesOrderHeader] AS [h]
ON [d].[SalesOrderID] = [h].[SalesOrderID]
WHERE [h].[TotalDue] > 500
AND [d].[ProductID] = 767
AND [d].[OrderQty] < 5;


/*
	Run PS script
	Then...troubleshot the stored procedure users are complaining about
*/
EXECUTE Sales.usp_GetProductInfo 712
GO



/*
	Try with different values
*/
EXECUTE Sales.usp_GetProductInfo 710
GO

EXECUTE Sales.usp_GetProductInfo 934
GO

EXECUTE Sales.usp_GetProductInfo 935
GO

/*
	Run these two together and compare cost
	check the compiled value
*/
EXECUTE Sales.usp_GetProductInfo 712
GO
EXECUTE Sales.usp_GetProductInfo 710
GO

/*
	Check the plan cache for fun...
*/
SELECT	[cp].[size_in_bytes],
		[cp].[cacheobjtype],
		[cp].[objtype],
		[cp].[plan_handle],
		[dest].[text],
		[qp].*
FROM [sys].[dm_exec_cached_plans] [cp]
CROSS APPLY [sys].[dm_exec_sql_text]([cp].[plan_handle]) [dest]
CROSS APPLY [sys].[dm_exec_query_plan] ([cp].[plan_handle]) [qp]
WHERE [dest].[text] LIKE '%usp_GetProductInfo%';


/*
	check compatability level
*/
SELECT [name], [compatibility_level]
FROM [sys].[databases]
WHERE [name] = 'AdventureWorks2014';

/*
	run a query
*/
SELECT
	[LastName],
	[FirstName],
	[PersonType],
	[ModifiedDate]
FROM [Person].[Person]
WHERE [LastName] = 'Stewart';

/*
	run with TF
*/
SELECT
	[LastName],
	[FirstName],
	[PersonType],
	[ModifiedDate]
FROM [Person].[Person]
WHERE [LastName] = 'Stewart'
OPTION (QUERYTRACEON 9481);

/*
	change compat level
*/
USE [master];
GO
ALTER DATABASE [AdventureWorks2014] SET COMPATIBILITY_LEVEL = 110;
GO

/*
	re-run
*/
USE [AdventureWorks2014];
GO

SELECT
	[LastName],
	[FirstName],
	[PersonType],
	[ModifiedDate]
FROM [Person].[Person]
WHERE [LastName] = 'Stewart';

/*
	use other TF
*/	
SELECT
	[LastName],
	[FirstName],
	[PersonType],
	[ModifiedDate]
FROM [Person].[Person]
WHERE [LastName] = 'Stewart'
OPTION (QUERYTRACEON 2312);


/*
	reset
*/
USE [master];
GO
ALTER DATABASE [AdventureWorks2014] SET COMPATIBILITY_LEVEL = 120;
GO